<?php
/**
 * @package     com_easyshop
 * @version     1.2.3
 * @Author      JoomTech Team
 * @copyright   Copyright (C) 2015 - 2019 www.joomtech.net All Rights Reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */

namespace ES;

defined('_JEXEC') or die;
use Joomla\Registry\Registry;

\JLoader::register('EasyshopHelper', ES_COMPONENT_ADMINISTRATOR . '/helpers/easyshop.php');

class CustomField
{
	protected $reflector;
	protected $reflector_id = 0;
	protected static $reflectors = [
		'com_easyshop.user',
		'com_easyshop.order',
		'com_easyshop.checkout',
		'com_easyshop.product.customfield',
		'com_easyshop.product.option',
		'com_easyshop.order.billing_address',
		'com_easyshop.order.shipping_address',
	];

	public function __construct(Array $config = [])
	{
		$this->setUp($config);
	}

	public function setUp(Array $config = [])
	{
		if (array_key_exists('reflector', $config))
		{
			$this->reflector = $config['reflector'];
		}

		if (array_key_exists('reflector_id', $config))
		{
			$this->reflector_id = (int) $config['reflector_id'];
		}
	}

	public function check($full = false, $reflector = null)
	{
		if (!$this->isValidReflector($reflector)
			|| ($full && (int) $this->reflector_id < 1)
		)
		{
			throw new \RuntimeException(\JText::_('COM_EASYSHOP_WARNING_PROVIDE_VALID_REFLECTOR'), 404);
		}

		return true;
	}

	public function load($key = 'id')
	{
		$this->check();
		$ref = $this->reflector;
		static $fields = [];

		if (!isset($fields[$ref]))
		{
			$showOn = (int) easyshop('app')->getClientId();
			$db     = easyshop('db');
			$query  = $db->getQuery(true)
				->select('a.id, a.name, a.alias, a.type, a.default_value, a.required, a.params, ' .
					'a.ordering, a.reflector, a.rules, a.showon, a.attributes, a.group_id, a.checkout_field, ' .
					'a.field_name, g.title AS group_title')
				->from($db->qn('#__easyshop_customfields', 'a'))
				->select('g.title AS group_title')
				->leftJoin($db->qn('#__categories', 'g') . ' ON a.group_id = g.id')
				->where('a.state = 1 AND a.showon IN (-1,' . $showOn . ')')
				->where('a.reflector = ' . $db->q($ref))
				->order('a.ordering ASC');
			$db->setQuery($query);
			$fields[$ref] = [];

			if ($rows = $db->loadObjectList($key))
			{
				foreach ($rows as $id => $row)
				{
					$this->parseLabelLanguage($row->name);
					$fields[$ref][$row->group_id][$id] = $row;
				}
			}
		}

		return $fields[$ref];
	}

	public function findField($id)
	{
		$groups = $this->load();

		foreach ($groups as $groupId => $fields)
		{
			foreach ($fields as $field)
			{
				if ($id == $field->id)
				{
					return $field;
				}
			}
		}

		return false;
	}

	protected function parseLabelLanguage(&$label)
	{
		$key = 'COM_EASYSHOP_CUSTOMFIELD_' . strtoupper($label);
		$key = preg_replace('/[^A-Z0-9]+/i', '_', $key);

		if (\JFactory::getLanguage()->hasKey($key))
		{
			$label = \JText::_($key);
		}

		return $label;
	}

	public function findFieldByName($fieldName)
	{
		foreach ($this->load() as $group => $fields)
		{
			foreach ($fields as $field)
			{
				if ($field->field_name == $fieldName)
				{
					return $field;
				}
			}
		}

		return false;
	}

	public function getItemsByGroup($groupId = 0, $reflectorId = 0)
	{
		$values = $this->getValues($reflectorId);
		$groups = $this->load();
		$items  = [];

		if (isset($groups[$groupId]))
		{
			foreach ($groups[$groupId] as $id => $item)
			{
				$items[$id]          = clone $item;
				$items[$id]->value   = null;
				$items[$id]->display = null;

				if (isset($values[$id]))
				{
					$items[$id]->value   = $values[$id]->value;
					$items[$id]->display = $values[$id]->display;
				}
				elseif ($items[$id]->type != 'checkbox')
				{
					$items[$id]->value   = $items[$id]->default_value;
					$items[$id]->display = $items[$id]->value;
				}
			}
		}

		easyshop('prepare.customfield', $items);

		return $items;
	}

	public function getValues($reflectorId = 0, $forceNew = false)
	{
		$this->check();
		static $values = [];
		$ref   = $this->reflector;
		$refId = (int) $reflectorId;

		if ($refId < 1)
		{
			$refId = $this->reflector_id;
		}

		if ($refId < 1)
		{
			return [];
		}

		if (!isset($values[$ref][$refId]) || $forceNew)
		{
			$db    = easyshop('db');
			$query = $db->getQuery(true)
				->select('a.customfield_id, a.reflector_id, a.value, a2.type, a2.name, a2.field_name, a2.alias, a2.params')
				->from($db->qn('#__easyshop_customfield_values', 'a'))
				->innerJoin($db->qn('#__easyshop_customfields', 'a2') . ' ON a2.id = a.customfield_id')
				->where('a.reflector LIKE ' . $db->q($ref . '%'))
				->where('a.reflector_id = ' . $refId)
				->order('a2.ordering ASC');
			$db->setQuery($query);

			if (!($fieldValues = $db->loadObjectList('customfield_id')))
			{
				$fieldValues = [];
			}

			easyshop('prepare.' . $this->reflector . 'Value', $fieldValues);

			foreach ($fieldValues as $field)
			{
				$this->parseLabelLanguage($field->name);
				$this->parseFieldValues($field);
			}

			$values[$ref][$refId] = $fieldValues;
		}

		return $values[$ref][$refId];
	}

	protected function parseFieldValues($field)
	{
		$type = strtolower($field->type);

		switch ($type)
		{
			case 'list':
			case 'checkbox':
			case 'checkboxes':
			case 'colors':
			case 'inline':
				$params      = $field->params;
				$renderer    = easyshop('renderer');
				$colorLabels = [];

				if (!($params instanceof Registry))
				{
					$params = new Registry($params);
				}

				$displayLayout = $params->get('displayLayout', 'text');

				if ($type == 'colors')
				{
					foreach ($params->get('options', []) as $colorOption)
					{
						$colorLabels[$colorOption->value] = $colorOption->text;
					}
				}

				if ($params->get('multiple') || $type === 'checkboxes')
				{
					if (is_string($field->value))
					{
						$field->value = explode('][', trim($field->value, '[]'));
					}

					if ($type == 'colors')
					{
						if ($displayLayout == 'text')
						{
							$field->display = array_map(function ($v) use ($colorLabels)
							{
								return isset($colorLabels[$v]) ? $colorLabels[$v] : $v;
							}, $field->value);

							$field->display = join(', ', $field->display);
						}
						else
						{
							$field->display = $renderer->render('form.field.color.display', ['colors' => $field->value]);
						}
					}
					else
					{
						$field->display = join(', ', $field->value);
					}
				}
				else
				{
					if ($type == 'colors')
					{
						if ($displayLayout == 'text')
						{
							$field->display = isset($colorLabels[$field->value]) ? $colorLabels[$field->value] : $field->value;
						}
						else
						{
							$field->display = $renderer->render('form.field.color.display', ['colors' => (array) $field->value]);
						}

					}
					else
					{
						$field->display = $field->value;
					}
				}

				break;

			case 'zone_country':
			case 'zone_state':
			case 'subzone':
				if ($zoneTable = easyshop('class.zone')->load($field->value))
				{
					$field->display = $zoneTable->name;
				}
				else
				{
					$field->display = $field->value;
				}
				break;

			case 'callname':
				$field->display = str_replace('][', ' ', trim($field->value, '[]'));
				break;

			case 'address':
				$field->display = str_replace('][', ', ', trim($field->value, '[]'));
				break;

			case 'ui_datetimepicker':
				/** @var \ES\Utility $utility */
				$utility        = easyshop('class.utility');
				$field->display = $utility->displayDate($field->value);

				break;

			default:
				$field->display = $field->value;
				break;
		}
	}

	public function parseFormFieldData($fieldsData)
	{
		$this->check();

		foreach ($fieldsData as $fieldId => $value)
		{
			/** @var $field \stdClass */
			if ($field = $this->findField($fieldId))
			{
				$fieldsData[$fieldId]        = clone $field;
				$fieldsData[$fieldId]->value = $value;
				$this->parseFieldValues($fieldsData[$fieldId]);
			}
			else
			{
				unset($fieldsData[$fieldId]);
			}
		}

		return $fieldsData;
	}

	public function insertValues(Array $data = [])
	{
		$columns = [
			'reflector',
			'reflector_id',
			'customfield_id',
			'value',
		];
		$values  = [];

		try
		{
			foreach ($data as $dt)
			{
				if (!array_key_exists('customfield_id', $dt))
				{
					throw new \RuntimeException('Invalid Data: customfield_id not found');
				}

				if (!array_key_exists('value', $dt))
				{
					throw new \RuntimeException('Invalid Data: value not found');
				}

				$customFieldId = (int) $dt['customfield_id'];
				$value         = $dt['value'];

				if (is_array($value))
				{
					$value = '[' . join('][', $value) . ']';
				}

				array_push($values, [$this->reflector, $this->reflector_id, $customFieldId, $value]);
			}
		}
		catch (\RuntimeException $e)
		{
			easyshop('app')->enqueueMessage($e->getMessage(), 'warning');

			return false;
		}

		if (count($values))
		{
			$this->removeValues();

			return easyshop('class.database')
				->insert('#__easyshop_customfield_values', $columns, $values);
		}

		return false;
	}

	public function register($nameString = null)
	{
		if (null !== $nameString)
		{
			$nameString = preg_replace('/[^z-zA-z0-9\.]/', '', $nameString);

			if (!in_array($nameString, self::$reflectors))
			{
				array_push(self::$reflectors, $nameString);
			}
		}

		return self::$reflectors;
	}

	public function isValidReflector($reflector = null)
	{
		if (null === $reflector)
		{
			$reflector = $this->reflector;
		}

		foreach (self::$reflectors as $rft)
		{
			if ($reflector == $rft && strpos($reflector, 'com_easyshop.') === 0)
			{
				return true;
			}
		}

		return false;
	}

	public function getFormFieldData($groupId = 0, Array $conditions = [], $group = 'customfields', $fieldName = null)
	{
		$data      = [];
		$items     = $this->getItemsByGroup($groupId);
		$xml       = new \DOMDocument('1.0', 'UTF-8');
		$form      = $xml->appendChild(new \DOMElement('form'));
		$fieldSets = $form->appendChild(new \DOMElement('fields'));
		$fieldSets->setAttribute('name', $group);
		$fieldSet = $fieldSets->appendChild(new \DOMElement('fieldset'));

		foreach ($items as $item)
		{
			$pass = true;

			foreach ($conditions as $k => $v)
			{
				if (!isset($item->{$k}) || $item->{$k} != $v)
				{
					$pass = false;

					break;
				}
			}

			if (!$pass)
			{
				continue;
			}

			if (empty($item->group_title))
			{
				$fieldsetName = str_replace('.', '_', $this->reflector);
			}
			else
			{
				$fieldsetName = preg_replace(['/\s+/', '/[^a-z0-9]/i'], ['_', ''], $item->group_title . '_' . $item->group_id);
			}

			if (empty($fieldsetName))
			{
				$fieldsetName = 'general';
			}

			$fieldSet->setAttribute('name', $fieldsetName);
			$fieldSet->setAttribute('label', $item->group_title);
			$name = $fieldName ? $fieldName . '_' . $item->id : $item->id;
			$this->prepareField($fieldSet, $item, $name);
			$data[$name] = $item->value;
		}

		return [
			'form' => new \SimpleXMLElement($xml->saveXML()),
			'data' => $data
		];
	}

	public function prepareField(\DOMElement $parentField, $itemField, $name)
	{
		$params      = new Registry($itemField->params);
		$field       = $parentField->appendChild(new \DOMElement('field'));
		$renderClass = trim($params->get('render_form_class'));
		$class       = 'field-' . $itemField->type . ' ' . $renderClass;
		$field->setAttribute('name', $name);
		$field->setAttribute('alias', $itemField->alias);
		$field->setAttribute('type', $itemField->type);
		$field->setAttribute('label', $itemField->name);
		$field->setAttribute('required', $itemField->required == '1' ? 'true' : 'false');
		$field->setAttribute('default', trim($itemField->default_value));
		$field->setAttribute('displayClass', trim($params->get('render_display_class')));

		if ($hint = $params->get('placeholder'))
		{
			$field->setAttribute('hint', trim($hint));
		}

		$attributes = trim($itemField->attributes);

		if (!empty($attributes))
		{
			$attributes = \JUtility::parseAttributes($attributes);

			if ($attributes)
			{
				$field->setAttribute('attributes', $attributes);
			}
		}

		if ($params->get('hiddenLabel'))
		{
			$field->setAttribute('hiddenLabel', 'true');
		}

		switch (strtolower($itemField->type))
		{
			case 'textarea':
				$field->setAttribute('rows', $params->get('rows', 5));
				$field->setAttribute('cols', $params->get('cols', 15));
				$class .= ' uk-textarea';
				break;

			case 'editor':
				$field->setAttribute('filter', 'safehtml');
				$field->setAttribute('buttons', 'true');
				$class .= ' uk-textarea';
				break;

			case 'text':
			case 'email':

				$class .= ' uk-input';

				if ($itemField->type == 'email')
				{
					$field->setAttribute('validate', 'email');
				}

				break;

			case 'callname':
				$field->setAttribute('filter', 'EasyshopHelper::filterArrayToString');
				$field->setAttribute('call_name_type', $params->get('call_name_type', 1));
				break;

			case 'address':
				$field->setAttribute('filter', 'EasyshopHelper::filterArrayToString');
				$field->setAttribute('address_line_2', $params->get('address_line_2', 1));
				break;

			case 'checkbox':
				$class .= ' uk-checkbox';
				break;

			case 'list':
			case 'radio':
			case 'checkboxes':
			case 'colors':
			case 'inline':

				if ($itemField->type == 'checkboxes')
				{
					$field->setAttribute('multiple', 'true');
				}
				else
				{
					$field->setAttribute('multiple', $params->get('multiple') ? 'true' : 'false');

					if ($itemField->type == 'list')
					{
						$class .= ' uk-select';
					}
				}

				$options = (array) $params->get('options', []);

				foreach ($options as $option)
				{
					$child = $field->appendChild(new \DOMElement('option'));
					$child->setAttribute('value', $option->value);
					$child->nodeValue = $option->text;
				}

				break;

			case 'zone_country':
			case 'zone_state':
			case 'subzone':
				$field->setAttribute('type', 'zone');
				$zoneType = str_replace('zone_', '', $itemField->type);
				$field->setAttribute('zone_type', $zoneType);
				$class .= ' uk-select';

				if ($itemField->type != 'zone_country')
				{
					$field->setAttribute('parent_id', '0');
				}

				break;

			case 'vat':
				$field->setAttribute('message', 'COM_EASYSHOP_CHECK_VAT_FAIL');
				$field->setAttribute('validate', 'vat');
				$class .= ' uk-input';
				break;

			case 'ui_datetimepicker':
				$field->setAttribute('filter', $params->get('filter', 'user_utc'));
				$field->setAttribute('todayButton', $params->get('todayButton', 'true'));
				$field->setAttribute('showTime', $params->get('showTime', 'true'));
				$field->setAttribute('changeMonth', $params->get('changeMonth', 'false'));
				$field->setAttribute('changeYear', $params->get('changeYear', 'false'));
				$field->setAttribute('yearRange', $params->get('yearRange', 'c-10:c+10'));
				$field->setAttribute('numberOfMonths', $params->get('numberOfMonths', '1'));
				$field->setAttribute('controlType', $params->get('controlType', 'slider'));
				break;
		}

		$class = array_unique(preg_split('/\s+/', $class));
		$field->setAttribute('class', trim(join(' ', $class)));
	}

	public function removeValues($reflector = 0, $reflectorId = 0)
	{
		if (!$reflector || !$reflectorId)
		{
			$this->check(true);

			if (!$reflector)
			{
				$reflector = $this->reflector;
			}

			if (!$reflectorId)
			{
				$reflectorId = $this->reflector_id;
			}
		}

		$db    = easyshop('db');
		$query = $db->getQuery(true)
			->delete($db->qn('#__easyshop_customfield_values'))
			->where($db->qn('reflector') . ' = ' . $db->q($reflector))
			->where($db->qn('reflector_id') . ' = ' . (int) $reflectorId);
		$db->setQuery($query);

		return $db->execute();
	}

	public function getGroups()
	{
		$this->check();
		static $groups = [];
		$ref = $this->reflector;

		if (!isset($groups[$ref]))
		{
			$db    = easyshop('db');
			$query = $db->getQuery(true)
				->select('a.id, a.title, a.alias, a.params')
				->from($db->qn('#__categories', 'a'))
				->where('a.published = 1')
				->where('a.extension = ' . $db->q($this->reflector))
				->order('a.lft ASC');
			$db->setQuery($query);

			if ($rows = $db->loadObjectList())
			{
				easyshop('prepare.category', $rows);
				$groups[$ref] = $rows;
			}
			else
			{
				$groups[$ref] = [];
			}
		}

		return $groups[$ref];
	}

	public function save($data = null, $groupName = 'customfields', $validate = false, $groupId = 0)
	{
		$this->check(true);
		$app = easyshop('app');

		if (!is_array($data))
		{
			$data = $app->input->get('jform', [], 'array');
		}

		if (empty($data[$groupName]))
		{
			return false;
		}

		if ($validate && $groupId)
		{
			/** @var $form \JForm */
			$formField = $this->getFormFieldData($groupId, [], $groupName);
			$form      = new \JForm('com_easyshop.customfield', ['control' => 'jform']);

			if (!$form->load($formField['form']))
			{
				return false;
			}

			$fieldsData = $form->filter($data, $groupName);
			$valid      = $form->validate($fieldsData, $groupName);

			if ($valid instanceof \Exception)
			{
				$app->enqueueMessage($valid->getMessage(), 'warning');

				return false;
			}

			if ($valid === false)
			{
				foreach ($form->getErrors() as $message)
				{
					$app->enqueueMessage($message, 'warning');
				}

				return false;
			}
		}
		else
		{
			$fieldsData = $data[$groupName];
		}

		$valueData = [];

		foreach ($fieldsData as $customFieldId => $value)
		{
			$valueData[] = [
				'customfield_id' => $customFieldId,
				'value'          => $value,
			];
		}

		return $this->insertValues($valueData);
	}
}
