<?php
/**
 * @package     com_easyshop
 * @version     1.2.3
 * @Author      JoomTech Team
 * @copyright   Copyright (C) 2015 - 2019 www.joomtech.net All Rights Reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */

namespace ES;

use Joomla\Registry\Registry;

defined('_JEXEC') or die;

class Order
{
	protected $address = null;
	protected $products = null;
	protected $checkoutFields = null;
	protected $customerName = null;
	protected $table = null;
	protected $subTotal = 0.00;

	/**
	 * @var $currency \ES\Currency
	 * @since 1.0.0
	 */
	public $currency = null;

	public function __construct($config = [])
	{
		\JTable::addIncludePath(ES_COMPONENT_ADMINISTRATOR . '/tables');
		$this->table    = \JTable::getInstance('Order', 'EasyshopTable');
		$this->currency = clone easyshop('class.currency')->getDefault();
	}

	public function getTable()
	{
		return $this->table;
	}

	public function get($name, $default = null)
	{
		$value = $this->__get($name);

		return $value === null ? $default : $value;
	}

	public function load($orderId)
	{
		$result  = false;
		$orderId = (int) $orderId;

		if ($orderId < 1)
		{
			return false;
		}

		if ($this->table->load($orderId))
		{
			foreach ($this->table->getProperties() as $field => $value)
			{
				$this->{$field} = $value;
			}

			$result = true;
			$this->currency->load($this->get('currency_id'));

			// Reset protected properties which would be been callback by the magic method
			$this->address        = null;
			$this->products       = null;
			$this->checkoutFields = null;
			$this->customerName   = null;
		}

		return $result;
	}

	public function getId()
	{
		$orderId = $this->get('id', 0);

		if (!$orderId)
		{
			throw new \RuntimeException(\JText::sprintf('COM_EASYSHOP_ERROR_ORDER_NOT_FOUND', $orderId));
		}

		return $orderId;
	}

	public function getCheckoutFields($orderId = 0)
	{
		if (!$orderId)
		{
			$orderId = $this->getId();
		}
		/** @var $customField \ES\CustomField */
		$customField = easyshop('class.customField', [
			'reflector' => 'com_easyshop.checkout',
		]);

		$fieldsValues = $customField->getValues($orderId, true);

		return $fieldsValues;
	}

	public function getAddress($orderId = 0)
	{
		if (!$orderId)
		{
			$orderId = $this->getId();
		}
		/** @var $customField \ES\CustomField */
		$customField = easyshop('class.customField', [
			'reflector' => 'com_easyshop.order.billing_address',
		]);
		$billing     = $customField->getValues($orderId, true);
		$customField = easyshop('class.customField', [
			'reflector' => 'com_easyshop.order.shipping_address',
		]);
		$shipping    = $customField->getValues($orderId, true);

		return [
			'billing'  => $billing,
			'shipping' => $shipping,
		];
	}

	public function getProductDetails($orderId = 0)
	{
		if (!$orderId)
		{
			$orderId = $this->getId();
		}

		$db    = easyshop('db');
		$query = $db->getQuery(true)
			->select('a.id AS order_product_id, a.product_id, a.product_name, a.product_taxes, a.product_price, '
				. 'a.product_shipping, a.quantity, a.product_discount_incl, a2.vendor_id')
			->leftJoin($db->qn('#__easyshop_products', 'a2') . ' ON a2.id = a.product_id')
			->from($db->qn('#__easyshop_order_products', 'a'))
			->where('a.order_id = ' . (int) $orderId);
		$db->setQuery($query);

		if ($products = $db->loadObjectList())
		{
			$subTotal   = 0.00;
			$totalTaxes = 0.00;

			foreach ($products as &$product)
			{
				$product->quantity      = (int) $product->quantity;
				$product->product_price = (float) $product->product_price;
				$product->product_taxes = (float) $product->product_taxes;
				$product->total_price   = $product->product_price * $product->quantity;
				$subTotal               += $product->total_price;
				$totalTaxes             += $product->product_taxes * $product->quantity;
				$query->clear()
					->select('a.id, a.order_product_id, a.option_id, a.option_name, a.option_text, a.option_value, a.option_price')
					->from($db->qn('#__easyshop_order_product_options', 'a'))
					->where('a.order_product_id = ' . (int) $product->order_product_id);
				$db->setQuery($query);

				if ($options = $db->loadObjectList())
				{
					$product->options = $options;
				}
			}

			$this->subTotal = $subTotal;
			$totalPrice     = $subTotal + $totalTaxes
				+ (float) $this->get('total_fee', 0.00)
				+ (float) $this->get('total_shipping', 0.00)
				- (float) $this->get('total_discount', 0.00);
		}
		else
		{
			$totalPrice = $totalTaxes = 0.00;
		}

		// Update Price and Taxes (maybe)
		if ($totalPrice != (float) $this->get('total_price', 0.00))
		{
			$this->table->load($orderId);
			$this->table->set('total_price', $totalPrice);
			$this->table->set('total_taxes', $totalTaxes);
			$this->table->store();
		}

		return $products;
	}

	public function saveAddress($type, $address)
	{
		$orderId = $this->getId();
		/** @var $customField \ES\CustomField */
		$customField = easyshop('class.customField', [
			'reflector'    => 'com_easyshop.order.' . $type . '_address',
			'reflector_id' => $orderId,
		]);
		$customField->save(['customfields' => $address]);
	}

	public function saveCheckoutFields($fieldsData)
	{
		$orderId = $this->getId();
		/** @var $customField \ES\CustomField */
		$customField = easyshop('class.customField', [
			'reflector'    => 'com_easyshop.checkout',
			'reflector_id' => $orderId,
		]);
		$customField->save(['customfields' => $fieldsData]);
	}

	public function save(&$data)
	{
		try
		{
			$this->table->set('id', 0);
			$this->table->reset();
			$this->table->bind($data);
			$isNew = empty($data['id']);

			if ($result = $this->table->store())
			{
				$orderId  = (int) $this->table->get('id');
				$this->id = $orderId;
				$this->saveAddress('billing', $data['billing_address']);
				$this->saveAddress('shipping', $data['shipping_address']);

				if (!empty($data['checkoutFields']))
				{
					$this->saveCheckoutFields($data['checkoutFields']);
				}

				/** @var $db \JDatabaseDriver */
				$db           = easyshop('db');
				$productQuery = $db->getQuery(true)
					->insert($db->qn('#__easyshop_order_products'))
					->columns([
						'order_id',
						'product_id',
						'product_name',
						'product_taxes',
						'product_price',
						'product_discount_incl',
						'product_shipping',
						'quantity',
					]);
				$optionQuery  = $db->getQuery(true)
					->insert($db->qn('#__easyshop_order_product_options'))
					->columns([
						'order_product_id',
						'option_id',
						'option_name',
						'option_text',
						'option_value',
						'option_price',
					]);

				foreach ($data['items'] as $item)
				{
					$shipping = 0.00;
					$productQuery->clear('values')
						->values(
							$orderId . ','
							. (int) $item['product']->id . ','
							. $db->q($item['product']->name) . ','
							. $this->currency->convert((float) $item['saleTaxes']) . ','
							. $this->currency->convert((float) $item['salePrice']) . ','
							. $this->currency->convert((float) $item['discountAmount']) . ','
							. $shipping . ','
							. (int) $item['quantity']
						);

					if ($db->setQuery($productQuery)->execute() && !empty($item['options']))
					{
						$orderProductId = (int) $db->insertid();

						foreach ($item['options'] as $optionId => $option)
						{
							$optionQuery->clear('values')
								->values($orderProductId . ','
									. (int) $optionId . ','
									. $db->q($option['name']) . ','
									. $db->q($option['text']) . ','
									. $db->q($option['value']) . ','
									. $this->currency->convert((float) $option['prefix'])
								);
							$db->setQuery($optionQuery)
								->execute();
						}
					}
				}

				if (!empty($data['discounts']))
				{
					$discountIds = [];

					foreach ($data['discounts'] as $discount)
					{
						if ((int) $discount->type === 1 && !in_array((int) $discount->id, $discountIds))
						{
							$discountIds[] = (int) $discount->id;
						}
					}

					if ($discountIds)
					{
						$query = $db->getQuery(true)
							->delete($db->qn('#__easyshop_order_coupons'))
							->where($db->qn('order_id') . ' = ' . $orderId);
						$db->setQuery($query)
							->execute();

						$query->clear()
							->insert($db->qn('#__easyshop_order_coupons'))
							->columns($db->qn(['order_id', 'coupon_id', 'handled']));

						foreach ($discountIds as $discountId)
						{
							$query->values($orderId . ', ' . $discountId . ', 0');
						}

						$db->setQuery($query)
							->execute();
					}
				}

				$data['order_id']   = $orderId;
				$data['order_code'] = $this->table->get('order_code');
				$this->load($orderId);
			}
			else
			{
				throw new \RuntimeException(join("\n", $this->table->getErrors()));
			}
		}
		catch (\RuntimeException $e)
		{
			throw new \RuntimeException($e->getMessage());
		}

		$app = easyshop('app');
		$app->triggerEvent('onEasyshopAfterSave', ['com_easyshop.order', $this->table, $isNew, $data]);
		$app->triggerEvent('onEasyshopOrderAfterSave', [$this, $isNew]);

		return $result;
	}

	public function getPaymentStatus()
	{
		return [
			'0' => \JText::_('COM_EASYSHOP_PAYMENT_UNPAID'),
			'1' => \JText::_('COM_EASYSHOP_PAYMENT_PAID'),
			'2' => \JText::_('COM_EASYSHOP_PAYMENT_REFUND'),
		];
	}

	public function getOrderStatus()
	{
		return [
			'0'  => \JText::_('COM_EASYSHOP_ORDER_CREATED'),
			'1'  => \JText::_('COM_EASYSHOP_ORDER_CONFIRM'),
			'2'  => \JText::_('COM_EASYSHOP_ORDER_PROCESSED'),
			'3'  => \JText::_('COM_EASYSHOP_ORDER_SHIPPED'),
			'4'  => \JText::_('COM_EASYSHOP_ORDER_SUCCEED'),
			'5'  => \JText::_('COM_EASYSHOP_ORDER_CANCELLED'),
			'-2' => \JText::_('COM_EASYSHOP_ORDER_TRASH'),
		];
	}

	public function removeProduct($orderProductId = 0, $orderId = 0)
	{
		$orderProductId = (int) $orderProductId;
		$orderId        = (int) $orderId;
		$db             = easyshop('db');
		$query          = $db->getQuery(true);

		if ($orderId < 1)
		{
			$query->select('a.order_id')
				->from($db->qn('#__easyshop_order_products', 'a'))
				->where('a.id = ' . $orderProductId);
			$db->setQuery($query);
			$orderId = $db->loadResult();
		}

		$query->clear()
			->delete($db->qn('#__easyshop_order_products'))
			->where($db->qn('id') . ' = ' . $orderProductId);
		$db->setQuery($query)
			->execute();

		$query->clear()
			->delete($db->qn('#__easyshop_order_product_options'))
			->where($db->qn('order_product_id') . ' = ' . $orderProductId);
		$db->setQuery($query)
			->execute();

		if ($orderId)
		{
			$this->load($orderId);
		}

		return $this;
	}

	public function addProduct($productData, $options = [])
	{
		/**
		 * @var $db  \JDatabaseDriver
		 * @var $log \ES\Log
		 */
		$db         = easyshop('db');
		$dataObject = (object) $productData;

		if ((int) $dataObject->id > 0)
		{
			$result = $db->updateObject('#__easyshop_order_products', $dataObject, 'id');
		}
		else
		{
			$result = $db->insertObject('#__easyshop_order_products', $dataObject, 'id');
		}

		if ($result && $options)
		{
			$query = $db->getQuery(true)
				->delete($db->qn('#__easyshop_order_product_options'))
				->where($db->qn('order_product_id') . ' = ' . (int) $dataObject->id);
			$db->setQuery($query)
				->execute();

			foreach ($options as $option)
			{
				if (is_array($option) || is_object($option))
				{
					if (is_array($option))
					{
						$option = (object) $option;
					}

					$option->order_product_id = (int) $dataObject->id;
					$db->insertObject('#__easyshop_order_product_options', $option, 'id');
				}

			}
		}

		$this->load($dataObject->order_id);
		$this->updateProductStock();

		return $this;
	}

	public function getLayoutData($layoutId = null, $extraDisplayData = null, $extraLayoutPaths = null)
	{
		/**
		 * @var $renderer     \ES\LayoutHelper
		 * @var $utility      \ES\Utility
		 * @var $currency     \ES\Currency
		 */
		$orderId  = $this->getId();
		$renderer = easyshop('renderer');
		$utility  = easyshop('class.utility');
		$currency = easyshop('class.currency');
		$currency->load($this->get('currency_id'));
		$shippingId  = $this->get('shipping_id', 0);
		$paymentId   = $this->get('payment_id', 0);
		$displayData = [
			'order'         => $this,
			'currency'      => $currency,
			'utility'       => $utility,
			'orderStatus'   => $this->getOrderStatus(),
			'paymentStatus' => $this->getPaymentStatus(),
			'address'       => $this->get('address'),
			'config'        => easyshop('config'),
			'payment'       => null,
			'shipping'      => null,
		];
		$replaceData = [
			'{ORDER_ID}'         => $orderId,
			'{ORDER_CODE}'       => $this->get('order_code'),
			'{CUSTOMER_NAME}'    => $this->customerName,
			'{CUSTOMER_EMAIL}'   => $this->get('user_email'),
			'{NOTE}'             => $this->get('note'),
			'{BILLING_ADDRESS}'  => $utility->formatAddress($displayData['address']['billing']),
			'{SHIPPING_ADDRESS}' => $utility->formatAddress($displayData['address']['shipping']),
			'{CART_BODY}'        => $renderer->render('email.order-cart-body-html', $displayData),
			'{USER_NAME}'        => \JFactory::getUser()->name,
			'{ORDER_STATUS}'     => $displayData['orderStatus'][$this->get('state')],
			'{PAYMENT_STATUS}'   => $displayData['paymentStatus'][$this->get('payment_status')],
			'{SHIPPING_METHOD}'  => '',
			'{PAYMENT_METHOD}'   => '',
		];

		if ($shippingId || $paymentId)
		{
			$methodClass = easyshop('class.method');

			if ($shippingId && ($shipping = $methodClass->get($shippingId)))
			{
				$displayData['shipping']          = $shipping;
				$replaceData['{SHIPPING_METHOD}'] = $shipping->name;
			}

			if ($paymentId && ($payment = $methodClass->get($paymentId)))
			{
				$displayData['payment']          = $payment;
				$replaceData['{PAYMENT_METHOD}'] = $payment->name;
			}
		}

		if (easyshop('site'))
		{
			$paths = [
				JPATH_SITE . '/templates/' . $renderer->getSiteTemplate() . '/html/com_easyshop/layouts',
				ES_COMPONENT_ADMINISTRATOR . '/layouts'
			];
		}
		else
		{
			$paths = [
				JPATH_SITE . '/templates/' . easyshop('app')->getTemplate() . '/html/com_easyshop/layouts',
				ES_COMPONENT . '/layouts',
			];

		}

		if (null !== $extraLayoutPaths)
		{
			settype($extraLayoutPaths, 'array');
			$paths = array_merge($extraLayoutPaths, $paths);
		}

		if (null !== $extraDisplayData)
		{
			settype($extraDisplayData, 'array');
			$displayData = array_merge($displayData, $extraDisplayData);
		}

		$renderer->setPaths($paths);

		if ($layoutId)
		{
			$html = $renderer->render($layoutId, $displayData);

			foreach ($replaceData as $key => $value)
			{
				if (stripos($html, $key) !== false)
				{
					$html = str_ireplace($key, $value, $html);
				}
			}

			return $html;
		}

		return $replaceData;
	}

	public function updatePayment($status, Array $extraData = [])
	{
		$this->getId();

		if (!in_array((string) $status, array_keys($this->getPaymentStatus())))
		{
			return false;
		}

		$orderTable                  = $this->getTable();
		$extraData['payment_status'] = $status;

		foreach ($extraData as $name => $value)
		{
			$orderTable->set($name, $value);
			$this->{$name} = $value;
		}

		if ($orderTable->store())
		{
			return $this;
		}

		return false;
	}

	public function paid(Array $extraData = [])
	{
		if ($this->updatePayment(1, $extraData))
		{
			easyshop('app')->triggerEvent('onEasyshopOrderPaid', [$this]);
			easyshop('class.log')->addEntry('com_easyshop.order', 'COM_EASYSHOP_LOG_ORDER_PAID', [$this->get('order_code')]);
			$this->updateProductStock();
		}

		return $this;
	}

	public function unpaid(Array $extraData = [])
	{
		if ($this->updatePayment(0, $extraData))
		{
			easyshop('app')->triggerEvent('onEasyshopOrderUnpaid', [$this]);
			easyshop('class.log')->addEntry('com_easyshop.order', 'COM_EASYSHOP_LOG_ORDER_UNPAID', [$this->get('order_code')]);
		}

		return $this;
	}

	public function refund(Array $extraData = [])
	{
		if ($this->updatePayment(2, $extraData))
		{
			easyshop('app')->triggerEvent('onEasyshopOrderRefund', [$this]);
			easyshop('class.log')->addEntry('com_easyshop.order', 'COM_EASYSHOP_LOG_ORDER_REFUND', [$this->get('order_code')]);
		}

		return $this;
	}

	public function __get($name)
	{
		switch ($name)
		{
			case 'address':
			case 'products':
			case 'checkoutFields':

				if (null === $this->{$name})
				{
					$callBack      = [
						'address'        => 'getAddress',
						'products'       => 'getProductDetails',
						'checkoutFields' => 'getCheckoutFields',
					];
					$orderId       = $this->getId();
					$result        = call_user_func_array([$this, $callBack[$name]], [$orderId]);
					$this->{$name} = $result ?: [];
				}

				break;

			case 'customerName':

				if (null === $this->customerName)
				{
					$orderId = $this->getId();

					if (empty($this->address['billing']))
					{
						$this->address = $this->getAddress($orderId);
					}

					$billing     = $this->address['billing'];
					$customField = easyshop('class.customField', [
						'reflector'    => 'com_easyshop.user',
						'reflector_id' => $orderId,
					]);

					$fieldName = $customField->findFieldByName('user_name');

					if ($fieldName && isset($billing[$fieldName->id]))
					{
						$this->customerName = $billing[$fieldName->id]->display;
					}
				}

				break;
		}

		return property_exists($this, $name) ? $this->{$name} : null;
	}

	public function updateProductStock()
	{
		$config      = easyshop('config');
		$orderStatus = $config->get('stock_order_update', []);
		$paid        = $config->get('stock_paid_update', 1);

		if (empty($orderStatus)
			|| empty($this->products)
			|| !in_array($this->state, $orderStatus)
			|| ($paid && (int) $this->payment_status !== 1)
		)
		{
			return;
		}

		/** @var $logClass \ES\Log */
		\JTable::addIncludePath(ES_COMPONENT_ADMINISTRATOR . '/tables');
		$productTable = \JTable::getInstance('Product', 'EasyshopTable');
		$logClass     = easyshop('class.log');

		foreach ($this->products as $product)
		{
			if ($productTable->load($product->product_id))
			{
				$currentStock = (int) $productTable->stock;

				if ($currentStock !== -1)
				{
					$stock = $currentStock - (int) $product->quantity;
					$productTable->set('stock', $stock > 0 ? $stock : 0);

					if ($productTable->store())
					{
						$logClass->addEntry('com_easyshop.product', 'COM_EASYSHOP_PRODUCT_STOCK_UPDATED_FORMAT', [$productTable->name, $productTable->id, $stock], $currentStock, $stock);
					}
				}
			}
		}
	}

	/**
	 * Get order status text
	 * @since 1.0.1
	 */
	public function getStatusText()
	{
		$orderStatus = $this->getOrderStatus();

		return isset($this->state) ? $orderStatus[$this->state] : null;
	}

	/**
	 * Get order payment status text
	 * @since 1.0.1
	 */
	public function getPaymentText()
	{
		$orderStatus = $this->getPaymentStatus();

		return isset($this->payment_status) ? $orderStatus[$this->payment_status] : null;
	}

	/**
	 * @param integer $orderId
	 * @param integer $orderStatus
	 * @param integer $paymentStatus
	 *
	 * @since 1.1.0
	 * @return mixed
	 */
	public function updateCouponLimit($orderId, $orderStatus, $paymentStatus)
	{
		$orderId = (int) $orderId;
		$db      = easyshop('db');
		$query   = $db->getQuery(true)
			->select('a.coupon_id, a2.limit, a2.coupon_code, a2.params')
			->from($db->qn('#__easyshop_order_coupons', 'a'))
			->innerJoin($db->qn('#__easyshop_discounts', 'a2') . ' ON a2.id = a.coupon_id')
			->where('a2.limit <> -1 AND a2.limit > 0 AND a.handled = 0 AND a.order_id = ' . $orderId);
		$db->setQuery($query);

		if ($rows = $db->loadObjectList())
		{
			$thisStatus = (int) $orderStatus;
			$thisPaid   = (int) $paymentStatus == 1;
			$logClass   = easyshop('class.log');

			foreach ($rows as $row)
			{
				$params = new Registry;
				$params->loadString((string) $row->params);
				$orderStatus = $params->get('handle_on_order_status', []);
				$orderPaid   = $params->get('handle_on_order_paid', 0);

				if (($orderStatus && !in_array($thisStatus, $orderStatus))
					|| ($orderPaid && !$thisPaid)
				)
				{
					continue;
				}

				$oldLimit = (int) $row->limit;
				$couponId = (int) $row->coupon_id;
				$newLimit = $oldLimit - 1;

				if ($newLimit < 0)
				{
					$newLimit = 0;
				}

				$query->clear()
					->update($db->qn('#__easyshop_discounts'))
					->set($db->qn('limit') . ' = ' . $newLimit)
					->where($db->qn('id') . ' = ' . $couponId);
				$db->setQuery($query)
					->execute();

				$query->clear()
					->update($db->qn('#__easyshop_order_coupons'))
					->set($db->qn('handled') . ' = 1')
					->where($db->qn('order_id') . ' = ' . $orderId)
					->where($db->qn('coupon_id') . ' = ' . $couponId);
				$db->setQuery($query)
					->execute();

				$logClass->addEntry('com_easyshop.discount', 'COM_EASYSHOP_COUPON_UPDATE_LIMIT_FORMAT', [$row->coupon_code, $oldLimit, $newLimit], $oldLimit, $newLimit);
			}
		}
	}

	/**
	 * @param \ES\Currency $currency
	 *
	 * @since 1.1.0
	 */
	public function setCurrency(Currency $currency)
	{
		$this->currency = $currency;
	}
}
