<?php
/**
 * @package     com_easyshop
 * @version     1.2.3
 * @Author      JoomTech Team
 * @copyright   Copyright (C) 2015 - 2019 www.joomtech.net All Rights Reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */

namespace ES\Model;

use Joomla\Registry\Registry;
use Joomla\Utilities\ArrayHelper;
use Joomla\String\Inflector;

defined('_JEXEC') or die;

class ModelAdmin extends \JModelAdmin
{
	public function __construct(array $config)
	{
		$config['events_map'] = [
			'delete'       => 'easyshop',
			'save'         => 'easyshop',
			'change_state' => 'easyshop'
		];

		$config['event_before_delete'] = 'onEasyshopBeforeDelete';
		$config['event_after_delete']  = 'onEasyshopAfterDelete';
		$config['event_before_save']   = 'onEasyshopBeforeSave';
		$config['event_after_save']    = 'onEasyshopAfterSave';
		$config['event_change_state']  = 'onEasyshopChangeState';
		$config['dbo']                 = easyshop('db');
		parent::__construct($config);
	}

	public function getForm($data = [], $loadData = true)
	{
		/** @var $form \JForm */
		$name = $this->getName();
		$form = $this->loadForm($this->option . '.' . $name, $name, ['control' => 'jform', 'load_data' => $loadData]);

		if (empty($form))
		{
			return false;
		}

		return $form;
	}

	protected function loadForm($name, $source = null, $options = [], $clear = false, $xpath = false)
	{
		$options['control'] = ArrayHelper::getValue((array) $options, 'control', false);
		$hash               = md5($source . serialize($options));

		if (isset($this->_forms[$hash]) && !$clear)
		{
			return $this->_forms[$hash];
		}

		$reflection = new \ReflectionClass($this);
		$path       = dirname($reflection->getFileName());

		\JForm::addFormPath($path . '/forms');
		\JForm::addFieldPath($path . '/fields');
		\JForm::addRulePath($path . '/rules');

		try
		{
			$form = \JForm::getInstance($name, $source, $options, false, $xpath);

			if (!\JLanguageMultilang::isEnabled() && $form->getField('language'))
			{
				$form->setFieldAttribute('language', 'description', 'COM_EASYSHOP_ENABLE_LANGUAGE_FILTER_DESC');
				$form->setFieldAttribute('language', 'readonly', 'true');
				$form->setFieldAttribute('language', 'default', '*');
			}

			if (!empty($options['load_data']))
			{
				$data = $this->loadFormData();
				$app  = easyshop('app');
				$view = $this->getName();

				if ($this->getState($view . '.id', 0) == 0)
				{
					$filters = (array) $app->getUserState('com_easyshop.' . Inflector::getInstance()->toPlural($view) . '.filter');

					foreach ($filters as $property => $value)
					{
						if (!empty($value) && $form->getField($property))
						{
							$data[$property] = $value;
						}
					}
				}
			}
			else
			{
				$data = [];
			}

			$this->preprocessForm($form, $data);
			$form->bind($data);
		}
		catch (\Exception $e)
		{
			$this->setError($e->getMessage());

			return false;
		}

		$this->_forms[$hash] = $form;

		return $form;
	}

	protected function preprocessForm(\JForm $form, $data, $group = 'easyshop')
	{
		/** @var  $customField \ES\CustomField */
		$customField = easyshop('class.customField', [
			'reflector'    => $form->getName(),
			'reflector_id' => isset($data['id']) ? (int) $data['id'] : 0,
		]);

		if ($customField->isValidReflector())
		{
			$formData = $customField->getFormFieldData();

			if (!empty($formData['form'])
				&& $formData['form'] instanceof \SimpleXMLElement
			)
			{
				if ($form->load($formData['form']))
				{
					$form->bind(['customfields' => $formData['data']]);
				}
			}
		}

		$this->postFormHook($form, $data);
	}

	protected function postFormHook(\JForm $form, $data)
	{
		easyshop('app')->triggerEvent('onEasyshopPrepareForm', [$form, $data]);
	}

	public function getTable($type = null, $prefix = 'EasyshopTable', $config = [])
	{
		if (null === $type)
		{
			$type = ucfirst($this->getName());
		}

		$reflection = new \ReflectionClass($this);
		$path       = dirname(dirname($reflection->getFileName()));
		\JTable::addIncludePath(ES_COMPONENT_ADMINISTRATOR . '/tables');
		\JTable::addIncludePath($path . '/tables');

		return \JTable::getInstance($type, $prefix, $config);
	}

	protected function loadFormData()
	{
		$app  = easyshop('app');
		$name = $this->getName();
		$data = $app->getUserState($this->option . '.edit.' . $name . '.data', []);

		if (empty($data))
		{
			$item = new Registry($this->getItem());
			$data = $item->toArray();
		}

		$this->preprocessData($this->option . '.' . $name, $data);

		return $data;
	}

	protected function preprocessData($context, &$data, $group = 'easyshop')
	{
		easyshop('app')->triggerEvent('onEasyshopPrepareData', [$context, &$data]);
	}

	public function save($data)
	{
		if (!empty($data['params']))
		{
			$registry = new Registry;
			$registry->loadArray($data['params']);
			$data['params'] = (string) $registry->toString();
		}

		if ($result = parent::save($data))
		{
			$name   = $this->getName();
			$itemId = (int) $this->getState($this->getName() . '.id');

			if (!empty($data['customfields']))
			{
				/** @var $customField \ES\CustomField */
				$fieldReflector = isset($this->fieldReflector) ? $this->fieldReflector : $name;
				$customField    = easyshop('class.customField', [
					'reflector'    => 'com_easyshop.' . $fieldReflector,
					'reflector_id' => $itemId,
				]);

				if ($customField->isValidReflector())
				{
					$customField->save($data);
				}
			}

			$addOns = easyshop('class.addon')->getAddons($name);
			$jform  = easyshop('app')->input->get('jform', [], 'array');

			if (count($addOns))
			{
				$db        = $this->getDbo();
				$query     = $db->getQuery(true);
				$addonData = empty($jform['addon']) ? [] : $jform['addon'];

				if (empty($addonData))
				{
					$query->delete($db->qn('#__easyshop_params'))
						->where($db->qn('context') . ' LIKE ' . $db->q('%.' . $name) . ' AND ' . $db->qn('item_id') . ' = ' . $itemId);
					$db->setQuery($query)
						->execute();
				}
				else
				{
					$values = [];

					foreach ($addonData as $element => $array)
					{
						if (isset($addOns[$element]))
						{
							$context = $element . '.' . $name;
							$array   = $this->validate($addOns[$element], $array);
							$query->clear()
								->delete($db->qn('#__easyshop_params'))
								->where($db->qn('context') . ' = ' . $db->q($context) . ' AND ' . $db->qn('item_id') . ' = ' . $itemId);
							$db->setQuery($query)
								->execute();

							if (false === $array)
							{
								return false;
							}

							$addonData[$element] = $array;
							$registry            = new Registry;
							$registry->loadArray($array);
							$values[] = $db->q($context) . ',' . $itemId . ',' . $db->q($registry->toString());
						}
					}

					if (count($values))
					{
						$query->clear()
							->insert($db->qn('#__easyshop_params'))
							->columns(['context', 'item_id', 'data'])
							->values($values);
						$db->setQuery($query)
							->execute();
					}
				}

				easyshop('app')->triggerEvent('onEasyshopAddonAfterSave', ['com_easyshop.' . $name, $addonData, $itemId]);
			}
		}

		return $result;
	}

	public function setState($property, $value = null)
	{
		$name  = str_ireplace('EasyshopModel', '', strtolower(get_class($this)));
		$key   = 'model.' . $name . '.state.' . $property;
		$state = easyshop('state')->get($key, null);

		if (null !== $state)
		{
			$value = $state;
		}

		return $this->state->set($property, $value);
	}

	public function getState($property = null, $default = null)
	{
		$name  = str_ireplace('EasyshopModel', '', strtolower(get_class($this)));
		$key   = 'model.' . $name . '.state.' . $property;
		$state = easyshop('state')->get($key, null);

		if (null !== $state)
		{
			return $state;
		}

		return parent::getState($property, $default);
	}

	protected function canEditState($record)
	{
		return easyshop('class.user')->core('edit.state');
	}

	protected function canDelete($record)
	{
		return easyshop('class.user')->core('delete');
	}

	public function getItem($pk = null)
	{
		$item = parent::getItem($pk);

		if (!empty($item->id))
		{
			easyshop('app')->triggerEvent('onEasyshopModelGetItem', ['com_easyshop.' . strtolower($this->getName()), $item]);
		}

		return $item;
	}
}
